package cz.jzitnik.tui;

import cz.jzitnik.game.Block;
import lombok.AllArgsConstructor;
import org.jline.terminal.Terminal;

import java.util.List;

@AllArgsConstructor
public class ScreenRenderer {
    private SpriteList spriteList;
    private Terminal terminal;

    private int[] getPlayerCords(List<Block>[][] world) {
        for (int i = 0; i < world.length; i++) {
            for (int j = 0; j < world[i].length; j++) {
                if (world[i][j].stream().anyMatch(x -> x.getBlockId().equals("steve"))) {
                    return new int[]{j, i};
                }
            }
        }
        return null;
    }

    public void render(List<Block>[][] world) {
        StringBuilder main = new StringBuilder();
        main.append("\033[H\033[2J");

        int[] cords = getPlayerCords(world);
        if (cords == null) return;

        int playerX = cords[0];
        int playerY = cords[1];

        int terminalWidth = terminal.getWidth();
        int terminalHeight = terminal.getHeight();

        int[] data = ScreenMovingCalculationProvider.calculate(playerX, playerY, terminalHeight, terminalWidth, world[0].length, world.length);

        // Calculate visible area boundaries
        int startX = data[0];
        int endX = data[1];
        int startY = data[2];
        int endY = data[3];

        int visibleWidth = endX - startX;
        int visibleHeight = endY - startY;

        // If the width is odd, reduce viewXRadius by 1
        if (visibleWidth % 2 != 0) {
            endX = Math.max(startX, endX - 1);
        }
        /*if (visibleHeight % 2 != 0) {
            endY = Math.max(startY, endY - 1);
        }*/

        StringBuilder[] lines = new StringBuilder[(endY - startY) * 26];
        for (int i = 0; i < lines.length; i++) {
            lines[i] = new StringBuilder();
        }

        int counter = 0;
        for (int y = startY; y < endY; y++) {
            for (int x = startX; x < endX; x++) {
                List<Block> blocks = world[y][x];
                String sprite = SpriteCombiner.combineSprites(blocks.stream()
                        .map(block -> spriteList.getSprite(block.getSprite()).getSprite())
                        .toArray(String[]::new));

                String[] spriteLines = sprite.split("\n");

                for (int i = 0; i < spriteLines.length; i++) {
                    lines[counter * 25 + i].append(spriteLines[i]);
                }
            }
            counter++;
        }

        for (StringBuilder line : lines) {
            main.append(line.toString());
            main.append("\n");
        }

        System.out.println(main);
    }
}
